#!/bin/bash

# VERSION 	5.1
# DATE		2022/07/15
# functions for bash.
set -e

#3668 for NVIDIA® Jetson Xavier™ NX
#3636 for Jetson TX2 NX

#3448 for NVIDIA® Jetson Nano™ and Jetson Nano 2GB
#3489 for Jetson TX2i and Jetson TX2 4GB

#2888 for NVIDIA® Jetson AGX Xavier™
#3310 for original NVIDIA® Jetson™ TX2
#2180 for Jetson TX1

D_NX_W1="0x3633"
D_NX_W2="0x3836"
D_TX2_NX_W1="0x3633"
D_TX2_NX_W2="0x3633"

D_NANO_W1="0x3433"
D_NANO_W2="0x3834"
D_TX2I_W1="0x3433"
D_TX2I_W2="0x3938"

D_AGX_W1="0x3832"
D_AGX_W2="0x3838"

D_ORIN_W1="0x3733"
D_ORIN_W2="0x3130"

D_TX2_W1="0x3333"
D_TX2_W2="0x3031"

D_TX1_W1="0x3132"
D_TX1_W2="0x3038"

D_BD_W1=""
D_BD_W2=""

D_MACHINE_X86=0
D_MACHINE_TK1=11
D_MACHINE_TX1=12
D_MACHINE_TX2=13
D_MACHINE_XAVIER=14
D_MACHINE_NANO=15
D_MACHINE_NX=16
D_MACHINE_ORIN=17
D_MACHINE_CLARA=18
D_MACHINE_TX2_NX=19
D_MACHINE_TX2I=20
D_MACHINE=""

D_DEV_KMSG=/dev/kmsg

exec_gstcmd()
{
	if [ ${D_BACKGROUND} = 0 ];then
		run_cmd $@
	else
		run_cmd_bg $@
	fi
}

exec_gstcmd_kmsg()
{
	if [ ${D_BACKGROUND} = 0 ];then
		run_cmd2kmsg $@
	else
		run_cmd2kmsg_bg $@
	fi
}
clear_cache()
{
	echo_title "clear_cache"
	echo 1 > /proc/sys/vm/drop_caches
	sleep 1
	echo 2 > /proc/sys/vm/drop_caches
	sleep 1
	echo 3 > /proc/sys/vm/drop_caches
	sleep 1
	sync
	#echo 0 > /proc/sys/vm/drop_caches
	
}


write2kern_log()
{
	echo $@ > /dev/kmsg
}

write2kern_log_t()
{
	curr=$(print_time) 
	write2kern_log ${curr} $@ 
}

app2front()
{
	echo "app2front= $@"
	#if [ -n $1 ]; then 
		wmctrl -a $1
		#echo "app2front-wmctrl $1"
	#fi
	#root@tegra-ubuntu-tx2:/media/nvidia/ext4# 
}

get_time_str()
{
	DT=$(date '+%Y%m%d_%H%M%S')
	echo ${DT}
}

print_time()
{
	curr=$(date +%d%m%Y-%H:%M:%S)
	echo "[${curr}] $@"
}

#show title
echo_title()
{
	echo "=====================================" 
	echo $@ 
	echo "=====================================" 
}

echo_title_2kmsg()
{
	echo "=====================================" > ${D_DEV_KMSG}
	echo $@ > ${D_DEV_KMSG}
	echo "=====================================" >  ${D_DEV_KMSG}
}

echo_title_t()
{
	curr=$(print_time) 
	echo_title ${curr} $@ 
}

echo_title_t2kmsg()
{
	curr=$(print_time) 
	echo_title_2kmsg ${curr} $@ 
}

# print and execute command
run_cmd()
{
	echo $@ 
	$@
}

run_cmd2kmsg()
{
	echo $@ 
	echo_title_2kmsg $@
	$@
}

# print and execute command in background mode
run_cmd_bg()
{
	echo "$@ &" 
	$@ &
}

# print and execute command in background mode
run_cmd2kmsg_bg()
{
	echo "$@ &" 
	echo_title_2kmsg $@
	$@ &
}

# execute and print timestamp
run_cmd_t()
{
	print_time $@ 
	$@
}

# install package
# Return 1 if install package, 0 for not execute the install process.
PkgInstall()
{
 	if [ -z "$1" ]; then
		echo 0
		#exit 1
    fi

	# check wget is exist
	QUERY_PKG=$(sudo apt -qq list $1 2>/dev/null)
	
	if [[ $QUERY_PKG != *"[installed]"* ]]; then
		#only do the update when the package is not installed.
		#sudo apt-get update
		run_cmd sudo apt-get install -y $1

		#QUERY_PKG=`$QUERY_CMD`
		#if [[ $QUERY_PKG != *"[installed]"* ]]; then
		#	"Can't install the $1, please check your internet connection!"
		#	exit 1
		#fi
		echo 1
	else
		echo 2
	fi
	
}

# Creakte directory when not exists
mkDirectory()
{
	if [ ! -d $1 ] ;then
		sudo mkdir -p $1
	fi
}

# Install Bonjour packages
Bonjour()
{
	echo_title "Package for Bonjour"
	RET=$(PkgInstall avahi-daemon)
	RET=$(PkgInstall avahi-discover)
	RET=$(PkgInstall avahi-utils)
}

# mount hdd to fs
mount_hdd()
{
	mkDirectory $3
	run_cmd mount -t $1 $2 $3
}

# mount samba share folder to local fs
mount_smb()
{
	PkgInstall cifs-utils; 
	mkDirectory $6
	run_cmd mount -t $1 -o user=$2,password=$3,domain=$4 $5 $6
	#sudo mount -t cifs -o user=James_cheng,password=E2345678e*,domain=YUAN //James_cheng/W "${DIR_MOUNT}"
	ls $6
}

#Create ssh connection
ssh_connection()
{
	RET=$(PkgInstall sshpass)
	sshpass -p ${1} ssh -o StrictHostKeyChecking=no ${2}@${3}  
}

# Install Bonjour packages
pkg_ssh()
{
	echo_title "Package for ssh"
	RET=$(PkgInstall sshpass)
	RET=$(PkgInstall apt-utils)
}

get_cpu_no()
{
	#CPU_JOB_NUM=$(grep processor /proc/cpuinfo | awk '{field=$NF};END{print field+1}')
	CPU_JOB_NUM=`nproc`
	#CPU_JOB_NUM=1
	echo ${CPU_JOB_NUM}
}

# https://stackoverflow.com/questions/13160309/conversion-hex-string-into-ascii-in-bash-command-line
# str_hex2ascii 32315559 ==> 21UY
str_hex2ascii()
{
	echo "${1}" | xxd -r -p
	echo ""
}

# https://stackoverflow.com/questions/5724761/ascii-hex-convert-in-bash
# str_ascii2hex "YU12"  : 59553132
str_ascii2hex()
{
	a=${1}
	for ((i=0;i<${#a};i++));do printf %02X \'${a:$i:1};done
}

get_machine()
{
	MACHINE=$(uname -m)
	if [[ "${MACHINE}" =~ "x86" ]]; then
		D_MACHINE=${D_MACHINE_X86}
	else
		GetBoard
	fi
	
	#echo ${D_MACHINE}
}

# https://forums.developer.nvidia.com/t/whether-the-jetson-nx-device-has-a-unique-id/188072
# https://docs.nvidia.com/jetson/archives/l4t-archived/l4t-3261/index.html#page/Tegra%20Linux%20Driver%20Package%20Development%20Guide/jetson_eeprom_layout.html
GetBoard()
{

	D_BD_W1=$(i2cget -f -y 0 0x50 0x19 w)
	D_BD_W2=$(i2cget -f -y 0 0x50 0x1b w)	
	
	case ${D_BD_W1} in
	${D_NX_W1} )
		case ${D_BD_W2} in
		${D_NX_W2} )		D_MACHINE=${D_MACHINE_NX} ;;
		${D_TX2_NX_W2} )	D_MACHINE=${D_MACHINE_TX2_NX} ;;
		esac
		;;	
	
	${D_NANO_W1} )
		case ${D_BD_W2} in
		${D_NANO_W2} )		D_MACHINE=${D_MACHINE_NANO} ;;
		${D_TX2I_W2} )		D_MACHINE=${D_MACHINE_TX2I} ;;
		esac
		;;
	
	${D_AGX_W1} )			D_MACHINE=${D_MACHINE_XAVIER} ;;
	${D_ORIN_W1} )			D_MACHINE=${D_MACHINE_ORIN} ;;
	${D_TX2_W1} )			D_MACHINE=${D_MACHINE_TX2} ;;
	${D_TX1_W1} )			D_MACHINE=${D_MACHINE_TX1} ;;
	esac
	
	#echo ${D_BD_W1}
	#echo ${D_BD_W2}	
}
